unit unlock;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Slock, Spin, ExtCtrls;

type
  TfrmCreateUnlock = class(TForm)
    grpUnlockCode: TGroupBox;
    edtChallenge: TEdit;
    lblChallenge: TLabel;
    GroupBox1: TGroupBox;
    lblUnlock: TLabel;
    edtUnlock: TEdit;
    grpUnlockType: TGroupBox;
    rdbRegister: TRadioButton;
    rdbExtend: TRadioButton;
    speExtension: TSpinEdit;
    Label1: TLabel;
    edtPrivateKey: TEdit;
    pnlButtons: TPanel;
    btnClose: TButton;
    grpQuickUnlock: TGroupBox;
    chkQuickUnlock: TCheckBox;
    procedure rdbExtendClick(Sender: TObject);
    procedure rdbRegisterClick(Sender: TObject);
    procedure edtChallengeChange(Sender: TObject);
    procedure btnCloseClick(Sender: TObject);
    procedure chkQuickUnlockClick(Sender: TObject);
  private
    procedure CalculateUnlock;
    procedure CalculateQuickUnlock;
  end;

var
  frmCreateUnlock: TfrmCreateUnlock;

implementation

uses
     SlokUtil;
{$R *.DFM}

{*******************************************************************************
* Procedure : rdbExtendClick                                                   *
********************************************************************************
* Purpose   : Sets the 'unlock mode' radio button to extend mode               *
********************************************************************************
* Paramters : None                                                             *
********************************************************************************
* Returns   : None                                                             *
*******************************************************************************}
procedure TfrmCreateUnlock.rdbExtendClick(Sender: TObject);
begin
     // set the mode
     speExtension.Enabled := True;

     // recalculate the unlock code based on the new mode
     edtChallengeChange(Self);
end; {rdbExtendClick}

{*******************************************************************************
* Procedure : rdbRegisterClick                                                 *
********************************************************************************
* Purpose   : Sets the 'unlock mode' radio button to register mode             *
********************************************************************************
* Paramters : None                                                             *
********************************************************************************
* Returns   : None                                                             *
*******************************************************************************}
procedure TfrmCreateUnlock.rdbRegisterClick(Sender: TObject);
begin
     // set the mode
     speExtension.Enabled := False;

     // recalculate the unlock code based on the new mode
     edtChallengeChange(Self);
end; {rdbRegisterClick}

{*******************************************************************************
* Procedure : edtChallengeChange                                               *
********************************************************************************
* Purpose   : Calculates the unlock string when the challenge info changes     *
********************************************************************************
* Paramters : None                                                             *
********************************************************************************
* Returns   : None                                                             *
*******************************************************************************}
procedure TfrmCreateUnlock.edtChallengeChange(Sender: TObject);
begin
     if chkQuickUnlock.Checked then
     begin
          // calculate the challenge string and get the
          // unlock code
          CalculateQuickUnlock;
     end
     else
     begin
          // calculate the unlock string from the challenge string
          CalculateUnlock;
     end; {if}
end; {edtChallengeChange}

{*******************************************************************************
* Procedure : CalculateUnlock                                                  *
********************************************************************************
* Purpose   : Calculates the unlock string based on the challenge string, the  *
*             Application Private Key and the extension value and mode         *
********************************************************************************
* Paramters : None                                                             *
********************************************************************************
* Returns   : None                                                             *
*******************************************************************************}
procedure TfrmCreateUnlock.CalculateUnlock;
begin
     if rdbExtend.Checked then
     begin
          // we are creating an extension code
          edtUnlock.Text := GetUnlockString(edtChallenge.Text,
                                            edtPrivateKey.Text,
                                            speExtension.Value);
     end
     else
     begin
          // we are creating a registration code
          edtUnlock.Text := GetUnlockString(edtChallenge.Text,
                                            edtPrivateKey.Text,
                                            0);
     end;
end; {CalculateUnlock}

{*******************************************************************************
* Procedure : CalculateQuickUnlock                                             *
********************************************************************************
* Purpose   : Calculates a quick unlock string (using no machine options, and  *
*             therefore dependant only on the User Name abd the encryption key *
********************************************************************************
* Paramters : None                                                             *
********************************************************************************
* Returns   : None                                                             *
*******************************************************************************}
procedure TfrmCreateUnlock.CalculateQuickUnlock;
var
     ChallengeString: string;
begin
     // get the challenge string based on the private key and the user name
     ChallengeString := GetChallenge(edtPrivateKey.Text + edtChallenge.Text, 0);

     if rdbExtend.Checked then
     begin
          // create the unlock code from the calculated challenge string
          edtUnlock.Text := GetUnlockString(ChallengeString, edtPrivateKey.Text, speExtension.Value);
     end
     else
     begin
          // create the unlock code from the calculated challenge string
          edtUnlock.Text := GetUnlockString(ChallengeString, edtPrivateKey.Text, 0);
     end;
end; {CalculateQuickUnlock}

{*******************************************************************************
* Procedure : btnCloseClick                                                    *
********************************************************************************
* Purpose   : Closes the SUnlock application                                   *
********************************************************************************
* Paramters : None                                                             *
********************************************************************************
* Returns   : None                                                             *
*******************************************************************************}
procedure TfrmCreateUnlock.btnCloseClick(Sender: TObject);
begin
     Close;
end; {btnCloseClick}

procedure TfrmCreateUnlock.chkQuickUnlockClick(Sender: TObject);
begin
     if chkQuickUnlock.Checked then
     begin
          // we are using the 'quick unlock' codes where
          // only the RegName and the EncryptionKey are
          // necessary - update the labels accordingly
          lblChallenge.Caption := 'User Name:';
          edtChallenge.Text := 'Enter user name';
          //grpUnlockType.Enabled := False;
     end
     else
     begin
          // we are using the full unlock code with a
          // challenge string
          lblChallenge.Caption := 'Challenge String:';
          edtChallenge.Text := 'Enter challenge string';
          //grpUnlockType.Enabled := True;
     end;
end;

end.
